/********************************************************************/
/* KINEMATICS.C							    */
/*								    */
/* Routines for handling kinematics of final state phase space for  */
/* use with Monte Carlo integration MOCCA			    */
/*					Ayres Freitas, Mar 03, 2000 */
/*								    */
/* compile with  g++ -c kinematics.C				    */
/********************************************************************/

#include "kinematics.h"
#include <math.h>

#include <iostream.h>
#include <iomanip.h>

double sqr(register double x)
{
  return x*x;
}


void swap(vector4 a, vector4 b)
{
  double store;
  unsigned i;
  
  for(i = 0; i < 4; i++, a++, b++)
  {
    store = *a;
    *a = *b;
    *b = store;
  }
}
  

vector4 P(cvector4 v)
{
  static double r[4];
  r[0] = v[0];
  r[1] =-v[1];
  r[2] =-v[2];
  r[3] =-v[3];
  return r;
}


double Dot(cvector4 a, cvector4 b)
{
  return (a[0]*b[0] - a[1]*b[1] - a[2]*b[2] - a[3]*b[3]);
}


/* Boost 4-vector v from rest frame of 4-momentum p = (m gamma, -m gamma beta)
   (beta is 3-velocity) into lab frame. Result in vboost.
*/
void Boost(cvector4 v, cvector4 p, vector4 vboost)
{
  double mass = sqrt(p[0]*p[0]-p[1]*p[1]-p[2]*p[2]-p[3]*p[3]);
  double pv = (p[0]*v[0]+p[1]*v[1]+p[2]*v[2]+p[3]*v[3])/mass;
  double fac = (pv+v[0])/(p[0]+mass);
  
  vboost[0] = pv;
  vboost[1] = v[1] + fac*p[1];
  vboost[2] = v[2] + fac*p[2];
  vboost[3] = v[3] + fac*p[3];
}


#define EPS	(double)1.e-3

double PseudoDelta(Complex ps, double ms)
{
  return ((abs(ps-ms) < EPS) ? 1.0 : 0.0);
}

/*Complex Prop(double ps, double ms, double mg)
{
  if(mg > 0.0)
    return 1/(ps - Complex(ms,mg));
  else
    return sqrt(M_PI/mg) * PseudoDelta(ps, ms);
}*/

Complex Prop(Complex ps, Complex ms, Complex mg)
{
  if(mg.real() == 0.0)
    return 1/(ps - ms);
  else
    return sqrt(M_PI/mg.real()) * PseudoDelta(ps.real(), ms.real());
}


double lambda(double m1s, double m2s, double m3s)
{
  double x = (m1s+m2s-m3s);
  x = x*x - 4*m1s*m2s;
  if(x < 0.0)
    return 0.0;
  else
    return x;
}


/* Determine the momenta of the outgoing particles for the deacy process
   X(m) -> Y(m1) + Z(m2) when X is in the rest frame. costheta and phi stand 
   for the angle of p1 relative to a fixed coordinate system. Result in p1, p2.
*/
void TwoDecay(double m, double m1, double m2, double costheta, double phi,
		vector4 p1, vector4 p2, double *factor)
{
  if(m < m1+m2)
  {
    *factor = 0.0;
    return;
  }
  double p = sqrt(lambda(m*m, m1*m1, m2*m2)) / (2*m);
  double sintheta = sqrt(1-costheta*costheta);
  
  p1[0] = (m*m + m1*m1 - m2*m2) / (2*m);
  p1[1] = p * sintheta * cos(phi);
  p1[2] = p * sintheta * sin(phi);
  p1[3] = p * costheta;
  
  p2[0] = m - p1[0];
  p2[1] = -p1[1];
  p2[2] = -p1[2];
  p2[3] = -p1[3];
  
  *factor = 1/(16*M_PI*M_PI) * p/m;
}

double _TwoDecay(double m, double m1, double m2, double costheta, double phi,
		vector4 p1, vector4 p2)
{
  if(m < m1+m2)
    return 0.0;

  double p = sqrt(lambda(m*m, m1*m1, m2*m2)) / (2*m);
  double sintheta = sqrt(1-costheta*costheta);
  
/*  double m1s = m*m, m2s = m1*m1, m3s = m2*m2;
  double x = (m*m+m1*m1-m2*m2);
  cout << " " << p << " " << m << " " << m1 << " " << x << " " <<
	  x*x-4*m1s*m2s << " " << 
	  lambda(m*m, m1*m1, m2*m2) << endl;*/
  p1[0] = (m*m + m1*m1 - m2*m2) / (2*m);
  p1[1] = p * sintheta * cos(phi);
  p1[2] = p * sintheta * sin(phi);
  p1[3] = p * costheta;
  
  p2[0] = m - p1[0];
  p2[1] = -p1[1];
  p2[2] = -p1[2];
  p2[3] = -p1[3];
  
  return 1/(16*M_PI*M_PI) * p/m;
}


/* Split a massive 4-momentum k into two light-like momenta p1,p2
*/
void SplitMom(cvector4 k, vector4 p1, vector4 p2)
{
  double mass = sqrt(k[0]*k[0]-k[1]*k[1]-k[2]*k[2]-k[3]*k[3]);
  double pv, fac;
  
  pv = (k[0]+k[3])/2;
  fac = (pv+mass/2)/(k[0]+mass);
  p1[0] = pv;
  p1[1] = fac*k[1];
  p1[2] = fac*k[2];
  p1[3] = fac*k[3] + mass/2;

  pv = (k[0]-k[3])/2;
  fac = (pv+mass/2)/(k[0]+mass);
  p2[0] = pv;
  p2[1] = fac*k[1];
  p2[2] = fac*k[2];
  p2[3] = fac*k[3] - mass/2;
}


unsigned xcount = 0;

/* Evaluate the spinor product s of two spinors for light-like momenta a,b.
*/
Complex S(cvector4 x, cvector4 y)
{
  double psqrt;
  
  if((x[0]-x[1]) > (1e-12*x[0]) && (y[0]-y[1]) > (1e-12*y[0]))
  {				/* choose k0 along x-axis, k1 along y-axis */
    psqrt = sqrt((y[0]-y[1])/(x[0]-x[1]));
    return (Complex(x[2],x[3]) * psqrt - Complex(y[2],y[3]) / psqrt);
  }
  else
  {
    xcount++;
    return Complex(0,0);
  }
}

/* Evaluate the spinor product t of two spinors for light-like momenta a,b.
*/
Complex T(cvector4 x, cvector4 y)
{
  Complex s = S(y,x);
  return Complex(s.real(), -s.imag());
//  return s;
}
